<?php
/**
 * Эта библиотека предназначена для преобразования результатов
 * поиска в БД в многомерный массив и HTML список
 * 
 * Пример разметки списка с багами
 * <ul>
 * 		<li>
 * 			Описание бага
 * 			<ul>
 * 				<li>Комментарий 1</li>
 * 					Комментарий 2
 * 					<ul>
 * 						<li>Ответ на комментарий 2</li>
 * 					</ul>
 * 			</ul>
 * 		</li>
 * </ul>
 *
 * @version 1.0
 * @link http://www.simplecoding.org
 * @author Стаценко Владимир <vova_33@gala.net>
 */
class Table2Tree {

	private $CI = null;
	//при задании параметров в этом массиве можно использовать шаблон {depth}, он
	//будет заменен на текущее значение глубины рекурсии (уровень вложенности
	//комментариев)
	private $config = array(
		'listOpen' => '<ul>'
		,'listClose' => '</ul>'
		,'bugOpen' => '<li>'
		,'bugClose' => '</li>'
		,'commentOpen' => '<li>'
		,'commentClose' => '</li>'
	);
	
	function Table2Tree() {
		$this->CI =& get_instance();
	}

	/**
	 * Преобразует данные из таблицы с результатами поиска багов в многомерный массив
	 * 
	 * @param $data - таблица (массив) с найденными багами
	 * @return многомерный массив
	 */
	function getTree($data) {
		$sTree = $this->_convert2SimpleTree($data);
		foreach ($sTree as $i => $row) {
			if (!empty($row['comments'])) {
				$sTree[$i]['comments'] = $this->_nestedComments($row['comments'], null, 0);
			}
		}
		return $sTree;
	}
	
	/**
	 * Преобразует многомерный массив в HTML список
	 * 
	 * @param $tree - многомерный массив
	 * @param $config - массив с настройками отображения списков
	 * @return строка в HTML формате
	 */
	function getHTMLList($tree, $config = null) {
		if (!empty($config)) {
			foreach ($config as $key => $value) {
				$this->config[$key] = $value;
			}
		}
		$this->CI->load->library('parser');
		$html = $this->config['listOpen']."\n";
		foreach ($tree as $node) {
			$html .= $this->config['bugOpen'].$this->CI->parser->parse('bug_tpl', $node, TRUE)."\n";
			if (!empty($node['comments'])) {
				$html .= $this->_convert2HTMLList($node['comments'], 0)."\n";
			}
			$html .= $this->config['bugClose'];
		}
		return $html.$this->config['listClose'];
	}

	/**
	 * Преобразует многомерный массив с комментариями в HTML список.
	 * 
	 * @param $comments - массив с комментариями
	 * @param $config - массив с настройками отображения списков
	 * @return строка в HTML формате
	 */
	function getHTMLCommentsList($comments, $config = null) {
		if (!empty($config)) {
			foreach ($config as $key => $value) {
				$this->config[$key] = $value;
			}
		}
		$this->CI->load->library('parser');
		$html = $this->_convert2HTMLList($comments, 0);
		return $html;
	}
	
	/**
	 * Преобразует таблицу с результатами поиска по БД в двумерный массив
	 * (описание бага и вложенный массив с комментариями). 
	 * 
	 * @param $bugs - массив с результатами поиска по БД
	 * @return двумерный массив с теми же данными
	 */
	function _convert2SimpleTree($bugs) {
		$bugRes = array();
		$currentBugId = 0;
		$i = -1;
		$j = 0;
		foreach ($bugs as $bug) {
			if ($bug['id'] != $currentBugId) {
				$currentBugId = $bug['id'];
				$i++;
				//заполняем массив данными о баге
				$bugRes[$i]['id'] = $bug['id'];
				$bugRes[$i]['title'] = $bug['title'];
				$bugRes[$i]['uname'] = $bug['uname'];
				$bugRes[$i]['category_id'] = $bug['category_id'];
				$bugRes[$i]['description'] = $bug['description'];
				$bugRes[$i]['status'] = $bug['status'];
				$bugRes[$i]['bug_date'] = $bug['bug_date'];
				$bugRes[$i]['uemail'] = $bug['uemail'];

				//данные о категории
				$bugRes[$i]['link'] = $bug['link'];
				$bugRes[$i]['category'] = $bug['category'];
				
				//создаем массив с комментариями
				if (isset($bug['c_id']) && $bug['c_id'] != NULL) {
					$bugRes[$i]['comments'] = array();
					$j = 0;
					$bugRes[$i]['comments'][$j]['id'] = $bug['c_id'];
					$bugRes[$i]['comments'][$j]['uname'] = $bug['c_uname'];
					$bugRes[$i]['comments'][$j]['description'] = $bug['c_description'];
					$bugRes[$i]['comments'][$j]['comment_date'] = $bug['comment_date'];
					$bugRes[$i]['comments'][$j]['parent_id'] = $bug['parent_id'];
					$bugRes[$i]['comments'][$j]['bug_id'] = $bug['bug_id'];
					$bugRes[$i]['comments'][$j]['uemail'] = $bug['c_uemail'];
					$j++;
				}
			}
			else {
				if (isset($bug['c_id'])) {
					//добавляем новую запись в массив с комментариями
					$bugRes[$i]['comments'][$j]['id'] = $bug['c_id'];
					$bugRes[$i]['comments'][$j]['uname'] = $bug['c_uname'];
					$bugRes[$i]['comments'][$j]['description'] = $bug['c_description'];
					$bugRes[$i]['comments'][$j]['comment_date'] = $bug['comment_date'];
					$bugRes[$i]['comments'][$j]['parent_id'] = $bug['parent_id'];
					$bugRes[$i]['comments'][$j]['bug_id'] = $bug['bug_id'];
					$bugRes[$i]['comments'][$j]['uemail'] = $bug['c_uemail'];
					$j++;
				}
			}
		}
		return $bugRes;
	}
	
	/**
	 * Этот метод создает многомерный массив с комментариями
	 * (используется поле parent_id).
	 * Рекурсивный метод.
	 * 
	 * @param $comments - исходный массив с комментариями (одномерный)
	 * @param $parentId - текущий родительский элемент (для первого вызова равен NULL)
	 * @param $depth - текущая глубина рекурсии (для первого вызова равна 0)
	 * @return многомерный массив со вложенными комментариями
	 */
	function _nestedComments($comments, $parentId, $depth) {
		$commentsRes = array();
		$i = 0;
		foreach ($comments as $val) {
			if ($val['parent_id'] == $parentId) {
				$commentsRes[$i] = $val;
				$commentsRes[$i]['depth'] = $depth;
				$commentsRes[$i]['comments'] = $this->_nestedComments($comments, $val['id'], $depth + 1);
				$i++;
			}
		}
		return $commentsRes;
	}
	
	/**
	 * Этот метод создает древовидный список комментариев.
	 * Рекурсивный метод.
	 * 
	 * @param $bugComments - многомерный массив с комментариями
	 * @param $depth - глубина рекурсии (для первого вызова равна 0)
	 * @return строка в HTML формате со списком
	 */
	function _convert2HTMLList($bugComments, $depth) {
		$list = '';
		$list .= $this->config['listOpen']."\n";
		foreach ($bugComments as $comment) {
			$openTag = str_replace('{depth}', $depth, $this->config['commentOpen']);
			$list .= $openTag.$this->CI->parser->parse('comment_tpl', $comment, TRUE)."\n";
			if (!empty($comment['comments'])) {
				$list .= $this->_convert2HTMLList($comment['comments'], $depth + 1);
			}
			$list .= $this->config['commentClose']."\n";
		}
		$list .= $this->config['listClose'];
		return $list;
	}
}
?>