<?php
/**
 * Эта модель предназначена для работы с комментариями
 *
 * @version 1.0
 * @link http://www.simplecoding.org
 * @author Стаценко Владимир <vova_33@gala.net>
 */
class MComment extends Model {

	function MComment() {
		parent::Model();
	}

	/**
	 * Добавляет новый комментарий в БД
	 * 
	 * @param $commentData массив с данными комментария
	 * @return TRUE - если комментарий добавлен, FALSE - в противном случае.
	 */
	function addComment($commentData) {
		$qAdd = 'INSERT INTO comments VALUES (NULL, ?, ?, NOW(), ?, ?)';
		return $this->db->query($qAdd, array(
			$commentData['uname'],
			$commentData['description'],
			$commentData['parent_id'],
			$commentData['bug_id']
		));
	}
	
	/**
	 * Удаляет комментарий из БД (связанные комментарии
	 * удаляются автоматически движком MySQL)
	 * 
	 * @param $id - id комментария в таблице
	 * @return TRUE - если комментарий удален, FALSE - в противном случае.
	 */
	function delete($id) {
		if (is_integer($id) || $id <= 0) {
			return false;
		}
		$qDel = 'DELETE FROM comments WHERE id=?';
		return $this->db->query($qDel, array($id));
	}

	/**
	 * Добавляет комментарий в БД.
	 * @param $commentData - массив с данными комментария
	 * @return id нового комментария или FALSE при возникновении ошибки
	 */
	function addNewComment($commentData) {
		$qAddComment = 'INSERT INTO comments (uname, description, comment_date, parent_id, bug_id, uemail)'
			.' VALUES (?, ?, NOW(), ?, ?, ?)';
		$res = $this->db->query($qAddComment, array(
						$commentData['uname'],
						$commentData['description'],
						$commentData['parent_id'],
						$commentData['bug_id'],
						$commentData['uemail']
		));
		if ($res) {
			return $this->db->insert_id();
		}
		return $res;
	}
}
?>